using System;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Windows.Forms;

namespace OlFoldersEvents
{
	[ComImport(), ComVisible(false), 
	Guid("00063076-0000-0000-C000-000000000046"),
	InterfaceType(ComInterfaceType.InterfaceIsIDispatch)]
	public interface IFoldersEvents
	{
		[DispId(0x0000f001)] void FolderAdd(object folder); 
		[DispId(0x0000f002)] void FolderChange(object folder); 
		[DispId(0x0000f003)] void FolderRemove(); 
	}

	[ClassInterface(ClassInterfaceType.None)]
	public class FoldersEvents_SinkHelper : IFoldersEvents, System.IDisposable 
	{
		private bool disposed = false;
		private UCOMIConnectionPointContainer oConnPointContainer;
		private UCOMIConnectionPoint m_oConnectionPoint;
		private int m_Cookie;
		private object olFolderObj;
		private AddinModule module;
		private object olFolders = null;
		private bool connected = false;

		public FoldersEvents_SinkHelper(AddinModule module)
		{
			oConnPointContainer = null; m_oConnectionPoint = null;
			m_Cookie = 0; this.module = module;
			olFolderObj = null;
		}

		~FoldersEvents_SinkHelper() 
		{ 
			Free(); 
		} 

		public void Dispose() 
		{ 
			Free(); 
			GC.SuppressFinalize(this); 
		} 

		private void Free() 
		{ 
			if (!disposed) 
			{ 
				RemoveConnection(); 
				module = null; 
				disposed = true; 
			} 
		} 

		public object Folder
		{
			get
			{
				return olFolderObj;
			}
		}

		public bool Connected
		{
			get
			{
				return connected;
			}
		}

		public bool ConnectTo(object olFolder) 
		{
			IntPtr unkObj = IntPtr.Zero;
			IntPtr unkFolder = IntPtr.Zero;

			if (module.HostType != AddinExpress.MSO.ADXOfficeHostApp.ohaOutlook) 
				return false;
			RemoveConnection();
			try
			{
				olFolderObj = olFolder;
				Guid guid = new Guid("00063006-0000-0000-C000-000000000046");
				unkObj = Marshal.GetIUnknownForObject(olFolderObj);
				Marshal.QueryInterface(unkObj, ref guid, out unkFolder);
				if (unkFolder != IntPtr.Zero)
				{
					olFolders = olFolderObj.GetType().InvokeMember("Folders", BindingFlags.GetProperty, null, olFolderObj, null);
					guid = new Guid("00063076-0000-0000-C000-000000000046");
					oConnPointContainer = (UCOMIConnectionPointContainer)olFolders;
					oConnPointContainer.FindConnectionPoint(ref guid, out m_oConnectionPoint);
					if (m_oConnectionPoint != null)
					{
						m_oConnectionPoint.Advise(this, out m_Cookie);
						connected = (m_Cookie > 0);
					}
					oConnPointContainer = null; 
				}
			}
			catch
			{
			}
			finally
			{
				if (unkObj != IntPtr.Zero)
					Marshal.Release(unkObj);
				if (unkFolder != IntPtr.Zero)
					Marshal.Release(unkFolder);
			}
			return connected;
		} 

		public void RemoveConnection() 
		{ 
			connected = false;
			if (m_Cookie != 0 && m_oConnectionPoint != null)
			{
				try
				{
					m_oConnectionPoint.Unadvise(m_Cookie);
				}
				catch
				{
				}
				m_Cookie = 0; olFolderObj = null;
				Marshal.ReleaseComObject(olFolders);
				Marshal.ReleaseComObject(m_oConnectionPoint);
			}
		} 

		void IFoldersEvents.FolderAdd(object folder) 
		{ 
			try
			{
				// Add some code
				MessageBox.Show("Folder Added.");
			}
			catch(Exception e)
			{
				MessageBox.Show(e.Message, e.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			finally
			{
				Marshal.ReleaseComObject(folder); 
			}
		} 

		void IFoldersEvents.FolderChange(object folder) 
		{ 
			try
			{
				// Add some code
				MessageBox.Show("Folder Changed.");
			}
			catch(Exception e)
			{
				MessageBox.Show(e.Message, e.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			finally
			{
				Marshal.ReleaseComObject(folder); 
			}
		} 

		void IFoldersEvents.FolderRemove() 
		{ 
			try
			{
				// Add some code
				MessageBox.Show("Folder Removed.");
			}
			catch(Exception e)
			{
				MessageBox.Show(e.Message, e.Source, MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
		}
	}
}
